#include "cons.hh"
#include "inputevent.hh"
#include "inputfield.hh"

class InputLine : public InputField
{
	Screen::ScreenRegion region;
	unsigned max, promptlen;
	unsigned curspos, xpos;
	bool insertmode;
	
public:
	InputLine(const char *fmt, va_list ap) : region(Display.AllocateLine(true)), curspos(0), xpos(0), insertmode(true)
	{
		unsigned len = region.printf(fmt, ap);
		
		promptlen = len;		
		max = region.xsize() - 1 - len;
		
		redraw();
	}
	InputLine(const char *fmt...) : region(Display.AllocateLine(true)), curspos(0), xpos(0), insertmode(true)
	{
		va_list ap;
		va_start(ap, fmt);
		unsigned len = region.printf(fmt, ap);
		va_end(ap);
		
		promptlen = len;		
		max = region.xsize() - 1 - len;
		
		redraw();
	}
	~InputLine()
	{
		Display.UnderlineCursor();
	}
	virtual void SetResult(const string &s)
	{
		result = s;
		redraw();
	}
	virtual void Handle(const InputEvent &ev)
	{
		switch(ev.type)
		{
			case inputchar:
				if(!insertmode)result.erase(curspos, 1);
				result.insert(curspos, 1, ev.character);
				++curspos;
				redraw();
				break;
			case inputkey:
    			switch(ev.key)
    			{
    				case KbLt:
    					if(curspos > 0) { --curspos; redraw(); }
    					break;
    				case KbRt:
    					if(curspos < result.size()) { ++curspos; redraw(); }
    					break;
    				case KbHome:
    					curspos = xpos = 0;
    					redraw();
    					break;
    				case KbEnd:
    					curspos = result.size();
    					redraw();
    					break;
    				case KbInsert:
    					insertmode = !insertmode;
    					break;
    				case KbBackSpace:
    					if(curspos > 0)--curspos;
    					// Fall thru
    				case KbDelete:
    					result.erase(curspos, 1);
    					redraw();
    					break;
    				default:
    					break;
    			}
    			break;
    		case inputctrl:
    			switch(ev.character)
    			{
    				case 'Y':
    					result = "";
    					curspos = 0;
    					xpos = 0;
    					redraw();
    					break;
    			}
			default:
				break;
		}
	}
	virtual void redraw()
	{
		if(curspos < xpos)xpos = curspos;
		else if(curspos >= xpos+max)xpos = curspos-max;
		
		region.gotox(promptlen);
		region.setattr(focused ? 0x1F : 0x07);
		region.printf("%s", result.substr(xpos, max).c_str());
		
		if(result.size()-xpos < max)
			region.putchar('_', max - (result.size()-xpos));
			
		if(insertmode)
			Display.UnderlineCursor();
		else
			Display.BlockCursor();
		
		region.gotox(promptlen + curspos-xpos);
		region.flush();
	}
};
