#include "cons.hh"

Screen::ScreenRegion::ScreenRegion
    (Screen *o, xycoord asize, xycoord aorig, bool temp)
    : owner(o), scrolling(false), temporary(temp),
      attr(0x07), orig(aorig), size(asize)
{
	pos.x = pos.y = 0;
	owner->allocate(orig, size);
	// fprintf(stderr, "Region: (%u,%u) at (%u,%u)\n", size.x,size.y, orig.x,orig.y);
	
	unsigned b = size.area();
	framebuffer = new BYTE[b * 2];
	for(unsigned ofs=0, a=0; a<b; ++a)
	{
		framebuffer[ofs++] = ' ';
		framebuffer[ofs++] = 0x07;
	}
}
Screen::ScreenRegion::~ScreenRegion()
{
	if(!owner)return;
	
	delete[] framebuffer;
	
	if(temporary)
	{
		setattr(0x07);
		for(unsigned y=0; y<ysize(); ++y)
		{
			gotoxy(0, y);
			putchar(' ', xsize());
		}		
	}
	owner->deallocate(orig, size, temporary);
}
unsigned Screen::ScreenRegion::printf(const char *fmt, va_list ap)
{
	char Buf[2048];
	unsigned b = vsnprintf(Buf, sizeof Buf, fmt, ap);
	puts(Buf);
	return b;
}
unsigned Screen::ScreenRegion::printf(const char *fmt ...)
{
	char Buf[2048];
	va_list ap;
	va_start(ap, fmt);
	unsigned b = vsnprintf(Buf, sizeof Buf, fmt, ap);
	va_end(ap);
	puts(Buf);
	return b;
}
void Screen::ScreenRegion::puts(const string &s, unsigned count)
{
	unsigned b = s.size();
	while(count > 0)
	{
		--count;
		for(unsigned a=0; a<b; ++a)
			putchar(s[a]);
	}
}
void Screen::ScreenRegion::putchar(char c, unsigned count)
{
	while(count > 0) { --count; putchar(c); }
}
void Screen::ScreenRegion::putchar(char c)
{
	switch((unsigned char)c)
	{
		case '\r':
			pos.x = 0;
			break;
		case '\n':
NewLine:	pos.x = 0;
			if(++pos.y >= size.y)
			{
				--pos.y;
				if(scrolling)
				{
					// Scrolling, TODO
				}
			}
			break;
		case '\b':
			if(pos.x > 0)--pos.x;
			break;
		case 32 ... 127:
		case 160 ... 254:
		{
			owner->Write(orig.x+pos.x, orig.y+pos.y, attr, c);
			if(++pos.x == xsize())
				goto NewLine;
		}
	}
}
void Screen::ScreenRegion::flush()
{
	owner->Write(orig.x+pos.x, orig.y+pos.y);
}
